

import java.util.ArrayList;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.util.Hashtable;	
import org.apache.commons.collections.map.MultiValueMap;

/**
 * An abstraction class for holding antonym definitions. 
 *
 * Constructed to read the file "anto.dat", which is a CSV-formatted file containg key,value-pairs.
 * The contents of this file is availiable for getting and setting via public methods.
 *
 * This class employs the MultiValueMap from org.apache.commons.collections.map to be able to map one word to different antonymes.
 *
 * @author Harald Svik
 * @see Syno
 *
 */

public class Anto {

	public File antofile;
	public BufferedReader d;
	public MultiValueMap ht = new MultiValueMap();


	/**
	 * Constructor for creating an Anto. Tries to access the file "anto.dat".
	 * If the files does not exist, it is created with default permissions. (Set by umask in UNIX environment)
	 *
	 */

	public Anto () {

		try { 
			this.antofile = new File("anto.dat");
			if ( ! antofile.exists() ) antofile.createNewFile();
			d = new BufferedReader(new FileReader(antofile));

			while (d.ready()) {
				String line = d.readLine();
				String[] parts = line.split(",",2);
				ht.put(parts[0],parts[1]);
			}

			System.out.println("== building of anto.dat complete ("+ht.totalSize()+" entries)");
			d.close();

		} catch ( Exception e ) {
			e.printStackTrace();
			System.out.println("== ERROR: could not open anto.dat");
			System.exit(1);
		}

	}

	/**
	 * Method to check if a word is an antonyme of another.
	 * Looks up all antonymes to the word in x, and then traverses all antonyme definitions.
	 * Returns true if Y is found. That is, true is the words are antonymes, false if the pair is not defined.
	 *
	 * @param x	the (gold) term to look up
	 * @param y the (test) term to compare
	 * @return true if the words are defined antonymes, else false
	 */

	public boolean isAnto(String x, String y){

		if ( ht.size(x) == 0) return false;
				  
		ArrayList<String> c = (ArrayList) ht.getCollection(x);
		// System.out.println(x+" collection is of size "+c.size());

		for ( String s : c ) {
			// System.out.println(x+" == "+s);
			if ( s.equals(y) ) return true;
		}

		// redundant
		return false;

	}

	/**
	 * Method to define two words as antonymes. Their sequence is not abitary: the first parameter is from GOLD and the second is from TEST.
	 * Writes (appends) the result to anto.dat, and assumes that the file is availiable. (If it was not, there would not have been an instance of Anto availiable.)
	 *
	 * @param x the GOLD word
	 * @param y the TEST word
	 *
	 */

	public void setAnto(String x, String y){

		try {

			FileWriter fw = new FileWriter(antofile,true); // true == append
			String towrite = x+","+y+"\n";
			// System.out.println("write: "+towrite);
			fw.write(towrite);
			fw.flush();
			fw.close();

		} catch ( Exception e ) {

			e.printStackTrace();
			System.out.println("== ERROR: failed to write to anto.dat");
			System.exit(1);

		}

		ht.put(x,y);

	}


}
