import java.io.*;
import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Iterator;

/** 
 * Main class for the EvEx program. Executed from the command line with the following syntax:
 * <ul>java EvEx [--debug {all,tql,ge,ontology}] [--manual] [--tqlxml file] [--genxml file] goldfile.xml testfile.xml
 * <li>--debug {all,tql,ge,ontology} takes a specification of which module to debug. If none if given, no debug information is printed.
 * <li>--manual specifies that the user can be bothered with question of biochemical character that will improve the evaluation
 * <li>--tqlxml file switches on writing of distilled events from the testfile to file
 * <li>--genxml file switches on writing of distilled events from the goldfile to file
 * <li>goldfile.xml defines which file to use as golden standard
 * <li>testfile.xml defines which file to read TQL-code from and compare against the golden standard
 * </ul>
 *
 * This program is also documented in the report and has a set of userdocs.
 *
 * @author      Harald Svik
 * @version     1.0
 * @since       1.0
 */
public class EvEx {


	public OntologyParser op = new OntologyParser("../data/genia20/GENIAevent.owl");
	public ArrayList<TQLAbstract> tqlabs = new ArrayList<TQLAbstract>();
	public ArrayList<GAbstract> genia = new ArrayList<GAbstract>();
	public ArrayList<Evaluator> evaluation = new ArrayList<Evaluator>();
	public File fileTestFile;
	public File fileGoldFile;

	public TQLParser tparser;
	public GENIAParser gparser;
	public Syno syno = new Syno();
	public Anto anto = new Anto();
	public String tqlxml_name;
	public String genxml_name;

	public boolean debug_tql;
	public boolean debug_ge;
	public boolean debug_op;
	public boolean manual = false;


	public EvEx(){

	}

	/**
	 * This is the invoking method of the entire system. It will
	 * <ul>
	 * <li>read the gold- and test file
	 * <li>invoke parsing of them
	 * <li>ensure that they are comparable
	 * <li>write destilled events to outfiles
	 * <li>start evaluation of each sentence
	 * <li>summarize evaluations
	 * <li>prettyprint the results
	 * </ul>
	 *
	 * @param strGoldFile a String that contains the path to the gold standard file
	 * @param strTestFile a String that contains the path to the file to be tested against the gold standard
	 *
	 *
	 */
	public void action(String strGoldFile, String strTestFile){

		try {
			fileTestFile = new File(strTestFile);
			tparser = new TQLParser(fileTestFile, op);
			tqlabs = tparser.getAbstracts();

			fileGoldFile = new File(strGoldFile);
			gparser = new GENIAParser(fileGoldFile);
			genia = gparser.getAbstracts();	

			// check file encodings
			FileReader fr = new FileReader(fileTestFile);
			FileReader gf = new FileReader(fileGoldFile);
			if( fr.getEncoding() != gf.getEncoding() ) System.out.println("== WARNING: "+fileTestFile.getName()+":"+fr.getEncoding()+", "+fileGoldFile.getName()+":"+gf.getEncoding());
		
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if ( tqlabs.size() == 0 ) {
				  System.out.println("No abstracts in TQL-code");
				  System.exit(1);
		} else if ( genia.size() == 0 ) {
				  System.out.println("No abstracts in GENIA-code");
				  System.exit(1);
		} else {

			XMLWriter xw = new XMLWriter(genxml_name, tqlxml_name);

			int tqlabs_size = tqlabs.size();
			int genabs_size = genia.size();
			int g_sent  = 0;	
			int t_sent  = 0;	
			int tql_events = 0;
			int gen_events = 0;

			for ( int k = 0 ; (k < genabs_size) && (k < tqlabs_size) ; k++ ) {

				TQLAbstract tqla = tqlabs.get(k);
				GAbstract gea = genia.get(k);

				tql_events += tqla.countEvents();
				t_sent += tqla.countTQLs();

				gen_events += gea.countEvents();
				g_sent += gea.countSentences();

				ArrayList gens = gea.getSentences();
				ArrayList tucs = tqla.getTQL();

				int i = tucs.size();
				int j = gens.size();

				if ( i > j ) {
					System.out.println(" == ERROR: More sentences in testfile than goldfile ("+i+" vs "+j+"). Manual correction required. Exiting.");
					System.exit(1);
				}

				int tql_pointer = 0;
				int gen_pointer = 0;

				while ( ( tql_pointer < tucs.size() ) && ( gen_pointer < gens.size() ) ) {

					TQL t = (TQL) tucs.get(tql_pointer);
					GSentence g = (GSentence) gens.get(gen_pointer);

					if ( t.input == g.sentence_number ) {
						Evaluator e = new Evaluator(g,t,this,syno,anto,xw);
						tql_pointer++;
						gen_pointer++;
						evaluation.add(e);

					} else {
						gen_pointer++;
					}
				}
			}

			xw.closeXML(); // no more output to files

			int g_total = 0;	
			int t_total = 0;	

			int t_type_total  = 0;
			int t_theme_total = 0;
			int t_cause_total = 0;

			int t_type_correct  = 0;
			int t_theme_correct = 0;
			int t_cause_correct = 0;
			int t_all_correct = 0;

			for ( Evaluator e : evaluation ) {
				g_total += e.g_total;
				t_total += e.t_total;

				// this is the number of EVALUATED attributes, which is < than number of guesses 
				// however, we should divide TP by NUMBER OF GUESSES (t_total)
				t_type_total  += e.t_type_total;
				t_theme_total += e.t_theme_total;
				t_cause_total += e.t_cause_total;

				t_type_correct  += e.t_type_correct;
				t_theme_correct += e.t_theme_correct;
				t_cause_correct += e.t_cause_correct;
				t_all_correct += e.t_all_correct;
			}

			System.out.println("\n ========= EXTRACTION RESULTS ========= \n");
			System.out.println("Total abstracts in file; \tTQL: "+tqlabs_size+", \tGE: "+genabs_size+"\t("+intDiv(tqlabs_size,genabs_size)+"%)");
			System.out.println("Events in all abstracts; \tTQL: "+tql_events+", \tGE: "+gen_events+"\t("+intDiv(tql_events,gen_events)+"%)");
			System.out.println("Sentences in all abstracts; \tTQL: "+t_sent+", \tGE: "+g_sent+"\t("+intDiv(t_sent,g_sent)+"%)");
			System.out.println("\n");

			tqlabs_size = 0;
			genabs_size = 0;
			tql_events = 0;
			gen_events = 0;
			t_sent = 0;
			g_sent = 0;

			int i_precision = intDiv((t_cause_correct+t_type_correct+t_theme_correct),(t_total*3));
			int i_recall = intDiv(t_total,g_total);

			float f_precision = (float) i_precision/100;
			float f_recall = (float) i_recall/100;

			float f_event_precision = (float) intDiv(t_all_correct,t_total)/100;
			float f_event_recall    = (float) intDiv(t_total,g_total)/100;

			System.out.println(" === PRECISION ===");
			System.out.println("Event precision:            \t\t"+ t_all_correct  +"/"+t_total      +"\t"+ perc(t_all_correct, t_total));
			System.out.println("Attribute precision (type): \t\t"+ t_type_correct +"/"+t_total +"\t"+perc(t_type_correct, t_total));
			System.out.println("Attribute precision (theme):\t\t"+ t_theme_correct+"/"+t_total +"\t"+perc(t_theme_correct,t_total));
			System.out.println("Attribute precision (cause):\t\t"+ t_cause_correct+"/"+t_total +"\t"+perc(t_cause_correct,t_total));
			System.out.println("Average precision:	\t\t"+ (t_type_correct+t_theme_correct+t_cause_correct)+"/3x"+t_total +"\t("+i_precision+"%)");
			System.out.println("\n");

			System.out.println(" === RECALL ===");
			System.out.println("Event recall:            \t\t"+ t_total  +"/"+g_total      +"\t"+ perc(t_total, g_total));
			//System.out.println("Attribute precision (type): \t\t"+ t_type_correct +"/"+t_total +"\t"+ perc(t_type_correct, t__total));
			//System.out.println("Attribute precision (theme):\t\t"+ t_theme_correct+"/"+t_total +"\t"+perc(t_theme_correct,t__total));
			//System.out.println("Attribute precision (cause):\t\t"+ t_cause_correct+"/"+t_total +"\t"+perc(t_cause_correct,t__total));
			System.out.println("\n");



			System.out.println(" === F-SCORE ===");
			// 2*P*R/(P+R)
			// 2*"+ f_precision +"*"+f_recall+"/"+f_precision+"+"+f_recall+"
			//  (0.75+1)*"+ f_precision +"/0.75+"+f_recall+"
			System.out.println("Harmonic F-score (attribute):           \t\t"+ ((2*f_precision*f_recall)/(f_precision+f_recall)));
			System.out.println("B-weighted F-score (attribute):         \t\t"+ ( (0.75*0.75 + 1) * f_precision )/(0.75*0.75 + f_recall));
			System.out.println("Harmonic F-score (event):            	\t\t"+ ((2*f_event_precision*f_event_recall)/(f_event_precision+f_event_recall)));
			System.out.println("B-weighted F-score (event):          	\t\t"+ (( (0.75*0.75 + 1) * f_event_precision )/(0.75*0.75 + f_event_recall)));
			System.out.println("\n");


			System.out.println("\n ========= EVALUATION COMPLETE ========= ");
			System.out.println("\n");

			/*
			System.out.println(" === WIDE EVALUATION RESULTS ===");
			System.out.println("Recall of events:\t\t"+t_total+"/"+g_total+"\t"+perc(t_total,g_total));
			System.out.println("Type precision: \t\t"+ t_type+"/"+ g_total+"\t"+perc(t_type, g_total));
			System.out.println("Theme precision:\t\t"+ t_theme+"/"+g_total+"\t"+perc(t_theme,g_total));
			System.out.println("Cause precision:\t\t"+ t_cause+"/"+g_total+"\t"+perc(t_cause,g_total));
			System.out.println("\n");

			System.out.println(" === NARROW EVALUATION RESULTS ===");
			System.out.println("Recall of events: \t\t"+t_total+"/"+t_total+"\t"+perc(t_total,t_total));
			System.out.println("Type precision: \t\t"+ t_type +"/"+t_total +"\t"+perc(t_type, t_total));
			System.out.println("Theme precision:\t\t"+ t_theme+"/"+t_total +"\t"+perc(t_theme,t_total));
			System.out.println("Cause precision:\t\t"+ t_cause+"/"+t_total +"\t"+perc(t_cause,t_total));
			System.out.println("\n");
			*/



			// System.out.println("Total overall precision: "+(t_type+t_theme+t_cause)/(g_total));

			/*
			ArrayList<Event> tevents = tqla.getEvents();
			for(Event e : tevents){
					  e.hello();
			}

			ArrayList<GEvent> gevents = gea.getEvents();
			for(GEvent e : gevents){
					  e.hello();
			}
			*/

			//comp = new Comparator(genia,tqlabs);
			//comp.evaluate();

		}

	}


	/**
	 * Invoked when starting program from command line, e.g.\ common Java behaviour. This method also takes care of parsing arguments from command line and set appropriate variables.
	 */
	public static void main(String[] args)
	{

		if(args.length < 2){
			System.err.println("usage: java EvEx [--debug {all,tql,ge,ontology}] [--manual] [--tqlxml file] [--genxml file] goldfile.xml testfile.xml");
			System.exit(1);
		} 

		String testfile = args[args.length-1];
		String goldfile = args[args.length-2];

		EvEx e = new EvEx();

		for ( int i = 0 ; i < args.length-2 ; i++ ) {

			if ( args[i].equals("--debug") ) {
				i++;
				if ( args[i].equals("all")){
					e.debug_tql = true;
					e.debug_ge = true;
					e.debug_op = true;
				} else if ( args[i].equals("tql") ) {
					e.debug_tql = true;
				} else if ( args[i].equals("ge") ) {
					e.debug_ge = true;
				} else if ( args[i].equals("ontology") ) {
					e.debug_op = true;
				} else {
					e.debug_tql = true;
					e.debug_ge = true;
					e.debug_op = true;
				}

			} else if ( args[i].equals("--manual") ) {
				e.manual = true;
			} else if ( args[i].equals("--tqlxml") ) {
				i++;
				e.tqlxml_name = args[i];
			} else if ( args[i].equals("--genxml") ) {
				i++;
				e.genxml_name = args[i];
			} else {
				System.err.println("== ERROR: parameter not recognized:"+args[i]+"\nusage: java EvEx [--debug {all,tql,ge,ontology}] [--manual] [--tqlxml outfile] [--genxml outfile] goldfile.xml testfile.xml");
				System.exit(1);
			}
		}

		e.action(goldfile,testfile);
	}

	/**
	 * Simplifies division of two integers by returning the results in percent.
	 * @param x dividend
	 * @param y divison
	 *
	 * @return the percent result
	 */
	public int intDiv(int x, int y){
		if ( y == 0 ) {
			System.err.println("== WARNING: Division by 0");
			return 0;
		} else {
			return (int) ( ( (double) x / (double) y ) *100);
		}
	}

	/**
	 * Wrapper method for intDiv that return the answer enclosed in a string, appended ( %)
	 * @param x divisor
	 * @param y dividend
	 * @return a string appended pretty characters
	 * @see #intDiv(int, int)
	 */
	public String perc(int x, int y) {
		return "("+intDiv(x,y)+"%)";
	}

	/**
	 * Method for float division that returns 0 if dividend is 0.
	 * @param x divisor
	 * @param y dividend
	 * @return the result
	 */
	public float floatDiv(float x, float y){
		if ( y == 0 ) {
			System.err.println("== WARNING: Division by 0");
			return 0;
		} else {
			return x/y;
		}
	}












}
/*
import java.io.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;


// Creation of an XML document
DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
DocumentBuilder db = dbf.newDocumentBuilder();
DOMImplementation di = db.getDOMImplementation();

Document doc = di.createDocument(null, "WFDB", null);
Element root = doc.getDocumentElement();

// This element will change in each line
Element el = null;

el = doc.createElement("WFDBann");
el.setAttribute("time",fieldInLine[0]);
root.appendChild(el);


// output of the XML document
DOMSource ds = new DOMSource(doc);
StreamResult sr = new StreamResult(new File(filename+".xml"));
TransformerFactory tf = TransformerFactory.newInstance();
Transformer trans = tf.newTransformer();
trans.transform(ds, sr);







*/
