import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * Standalone class for evaluation for XML-formatted event files.
 *
 * This class is meant to be invoked from the command line with two parameters: the gold file path and the test file path. If needed, it could also be instanciated as a Java object, but is primarily not written for this usage. The test files are a file with distilled events from EvEx. The syntax format for GOLD and TEST is equal.
 *
 * Distilled events are defined between <root> and </root>, one per line, in the format:
 * <event id='\d+' sentence='\d+' abstract='\d+'>  <type class='\w+' />  <theme idref='\w+' />  <cause idref='\w+' /></event>
 *
 * Events in this format is evaluated in a "stupid" fashion, e.g. no action is taken to make sure matching events are compared.
 *
 * @author Harald Svik
 * @see EvEx
 * @see Anto
 * @see Syno
 *
 */
public class EvalE {


	public File genfile;
	public File tqlfile;
	public EvalTHandler genhandler;
	public EvalTHandler tqlhandler;

	public Syno syno = new Syno();
	public Anto anto = new Anto();

	ArrayList<EvalEvent> tql_array = new ArrayList<EvalEvent>();
	ArrayList<EvalEvent> gen_array =	new ArrayList<EvalEvent>(); 

	/**
	 * The constructor of EvalE enables it to be invoked from another class if needed. Normally, it is invoked from command line.
	 *
	 * @param genfile_name the path to gold-file in distilled format
	 * @param tqlfile_name the path to test-file in distilled format
	 *
	 */
	public EvalE (String genfile_name, String tqlfile_name) {
		try {
			this.tqlfile = new File(tqlfile_name);
 			this.genfile = new File(genfile_name);		
		} catch ( Exception e ) {
			e.printStackTrace();
			System.exit(1);
		}
	}

	/**
	 * Static method allowing invocation of this class from command line.
	 */
	public static void main(final String[] args) {
		if ( args.length != 2) {
			System.out.println("usage: java EvalE gen.xml tql.xml");
			System.exit(1);
		}

	 	EvalE ee = new EvalE(args[0], args[1]);
		ee.eval();
	}

	/**
	 * The method for performing evaluation of events. The events are loaded sequentially, and a warning is printed if their number differ.
	 * There should be an equal number of events. If this is not the case, manual curation of event files are required.
	 *
	 */
	public void eval(){
		makeParser();	

		int tql_array_size = tql_array.size();
		int gen_array_size = gen_array.size();

		if ( tql_array_size != gen_array_size ) System.out.println("== WARNING: diffent number of events ("+tql_array_size+" vs. "+gen_array_size+")");

		int total = 0;
		int type  = 0;
		int theme = 0;
		int cause = 0;

		for ( int i = 0 ; i < tql_array_size ; i++ ) {

			EvalEvent t = tql_array.get(i);
			EvalEvent g = gen_array.get(i);

			total++;

			if ( intervention(t.type,g.type)   ) type++;
			if ( intervention(t.theme,g.theme) ) theme++;
			if ( intervention(t.cause,g.cause) ) cause++;


		}
		

		System.out.println("\n ========= EVALUATION RESULTS FOR SYNCHRONZED EVENTS ========= ");
		System.out.println("Type precision: \t\t"+type +"/"+total+"\t"+perc(type, total));
		System.out.println("Theme precision:\t\t"+theme+"/"+total+"\t"+perc(theme,total));
		System.out.println("Cause precision:\t\t"+cause+"/"+total+"\t"+perc(cause,total));

	}

	/**
	 * Intervention is a method for comparing two strings. It employs Syno and Anto to determine (in)equality beyond lexical comparision.
	 * If no specific similarity is found,  the user is requested to evaluate the event manually.
	 *
	 * @param ge the gold word
	 * @param te the test word
	 * @return result of the comparision
	 */
	public boolean intervention ( String ge, String te ) {

		if ( ( ge == null ) && ( te == null ) ) {
			return true;
		} else if ( ( ge == null ) || ( te == null ) ) {
			return false;
		} else if ( ge.equals(te) ) {
			return true;	
		} else if ( syno.isSyno(ge,te) ) {
			return true;	
		} else if ( anto.isAnto(ge,te) ) {
			return false;	
		}
		
		System.out.println("== manual intervention ==\n"+ge+" == "+te+" ? [0]");
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

		int i = 0;
		try {
			i = in.read();	
		} catch ( Exception e ) {
			e.printStackTrace();
		}
		if ( i == 49 ) {
			syno.setSyno(ge,te);
			return true;
		} else if ( i == 48 || i == 10) {
			anto.setAnto(ge,te);
			return false;
		} else {
			System.out.println("use (return, 0) for false or 1 for true");
			intervention(ge,te);
		}
		return false;
	}


	/**
	 * Method to invoke parsing of the xml event files.
	 */
	public void makeParser(){

		try {
			tqlhandler = new EvalTHandler(tql_array);
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser saxParser = factory.newSAXParser();
			saxParser.parse( tqlfile , tqlhandler );
		} catch ( Exception e ) {
			e.printStackTrace();
		}

		try {
			tqlhandler = new EvalTHandler(gen_array);
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser saxParser = factory.newSAXParser();
			saxParser.parse( genfile , tqlhandler );
		} catch ( Exception e ) {
			e.printStackTrace();
		}

	}


	/**
	 * Conatiner class to hold parsed events
	 */
	public class EvalEvent {

		public String type;
		public String theme;
		public String cause;

	}

	/**
	 * Internal class to parse the distilled xml event file. Adds EvalEvents to the ArrayList specified as member variable in EvaE.
	 *
	 */
	public class EvalTHandler extends DefaultHandler {

		EvalEvent currentEvent;
		ArrayList<EvalEvent> event_array;

		public EvalTHandler(ArrayList event_array) {
			this.event_array = event_array;
		}

		public void startDocument(){
			System.out.println("== starting eval parsing");
		}

		public void endDocument(){
			System.out.println("== eval parsing complete");
		}

		public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {
				
				if ( qName == "root" ) {

				} else if ( qName == "event" ) {
					currentEvent = new EvalEvent();
				} else if ( qName == "type" ) {
					currentEvent.type = attrs.getValue("class");	
				} else if ( qName == "theme" ) {
					currentEvent.theme = attrs.getValue("idref");	
				} else if ( qName == "cause" ) {
					currentEvent.cause = attrs.getValue("idref");	
				}
		}

		public void endElement(String namespaceURI, String sName, String qName ) throws SAXException {
				if ( qName == "root" ) {

				} else if ( qName == "event" ) {
					event_array.add(currentEvent);
					currentEvent = null;
				} else if ( qName == "type" ) {
				} else if ( qName == "theme" ) {
				} else if ( qName == "cause" ) {
				}
   	}

		public void characters(char buf[], int offset, int len) throws SAXException {
			String s = new String(buf, offset, len);
			s = s.trim();
  		}

		public void emit(String s) throws SAXException {
			if( s != null ) System.out.println(s);
		}

	}


	public int intDiv(int x, int y){
		if ( y == 0 ) {
			System.err.println("== WARNING: Division by 0");
			return 0;
		} else {
			return (int) ( ( (double) x / (double) y ) *100);
		}
	}

	public String perc(int x, int y) {
		return "("+intDiv(x,y)+"%)";
	}


}
