/**
 * Container class for holding events from GeneTUC.
 * Event are invoked from TQLHandler
 *
 * @see TQLHandler
 *
 */
public class Event {

	public int id;
	public String type;
	public String type_extended;
	public String theme;
	public String theme_extended;
	public String cause;
	public String cause_extended;
	public String cause_adjectives = "";
	public String litType;
	public String litTheme;
	public String litCause;
	public TQL parent;
	public TQLAbstract parent_abstract;


	/**
	 * Create an event with all attributes.
	 *
	 * @param p the parent TQL clause
	 * @param type the event type
	 * @param theme the event theme
	 * @param cause the event cause
	 *
	 */
	public Event (TQL p, String type, String theme, String cause) {
		this.type = type;
		this.theme = theme;
		this.cause = cause;
		this.parent = p;
		this.id = parent.getNextEvID();
	}

	/**
	 * Create a incomplete event for later refinement.
	 *
	 * @param p the parent TQL clause
	 * @param type the event type
	 * @param theme the event theme
	 *
	 */
	public Event (TQL p, String type, String theme) {
		this.type = type;
		this.theme = theme;
		this.parent = p;
		this.id = parent.getNextEvID();
	}

	/**
	 * Create a incomplete event for later refinement.
	 *
	 * @param p the parent TQL clause
	 * @param type the event type
	 *
	 */
	public Event (TQL p, String type) {
		this.type = type;
		this.parent = p;
		this.id = parent.getNextEvID();
	}

	/**
	 * Create a incomplete event for later refinement.
	 *
	 * @param p the parent TQL clause
	 *
	 */
	public Event (TQL p) {
		this.parent = p;
		this.id = parent.getNextEvID();
	}

	/**
	 * Create an empty event for filling "holes" in event arrays.
	 */
	public Event () {
		this.type = null;
		this.theme = null;
		this.cause = null;
		this.parent = null;
		this.id = 0;
	}

	/**
	 * Echos this event to stdout.
	 */
	public void printEvent(){
			  System.out.println("<Event: "+id+":"+parent.input+", type: "+this.getType()+" ("+litType+"), theme: "+this.getTheme()+" ("+litTheme+"), cause: "+this.getCause()+" ("+litCause+")>\n");
	}

	/**
	 * Wrapper method for printEvent();
	 * @see #printEvent()
	 */
	public void hello(){
			  printEvent();
	}

	/**
	 * Sets the type of this event
	 * @param t type
	 */
	public void setType(String t){
			  type = t;
			  type_extended = t;
	}

	/**
	 * Sets the theme of this event
	 * @param t theme
	 */
	public void setTheme(String t){
			  theme = t;
	}

	/**
	 * Sets the cause of this event
	 * @param c cause
	 */
	public void setCause(String c){
			  cause = c;
	}

	/**
	 * Gets the numeric ID of this event
	 * @return id
	 */
	public int getID(){
			  return id;
	}

	/**
	 * Gets the type of this event. There are a number of "types":
	 * <ul>
	 *  <li> type_extended are the semantic definition that most closely describes the type
	 *  <li> literal_type is the resolved skolemn for this type
	 *  <li> type is the original lexical representation of this type
	 * </ul>
	 *
	 * Types are prioritizedly returned (top-bottom).
	 *
	 * @return type
	 */
	public String getType(){
		if ( type_extended != null ) {
			return type_extended;
		} else if ( litType != null ) {
			return parent.sk(litType);
		} else {
			return type;
		}		
	}

	/**
	 * Gets the theme of this event. There are a number of "themes":
	 * <ul>
	 *  <li> theme_extended are the semantic definition that most closely describes the theme
	 *  <li> literal_theme is the resolved skolemn for this theme
	 *  <li> theme is the original lexical representation of this theme
	 * </ul>
	 *
	 * Types are prioritizedly returned (top-bottom).
	 *
	 * @return theme
	 */
	public String getTheme(){
		// System.out.println("THEME: "+theme+","+theme_extended+","+litTheme+","+parent.sk(litTheme));
		if ( theme_extended != null ) {
			return theme_extended;
		} else if ( litTheme != null ) {
			return parent.sk(litTheme);
		} else {
			return theme;
		}		
	}

	/**
	 * Gets the cause of this event. There are a number of "causes":
	 * <ul>
	 *  <li> cause_extended are the semantic definition that most closely describes the cause
	 *  <li> literal_cause is the resolved skolemn for this cause
	 *  <li> cause is the original lexical representation of this cause
	 * </ul>
	 *
	 * Types are prioritizedly returned (top-bottom).
	 *
	 * @return cause
	 */
	public String getCause(){
		if ( cause_extended != null ) {
			return cause_extended;
		} else if ( litCause != null ) {
			return parent.sk(litCause);
		} else {
			return cause;
		}		
	}

	/**
	 * Get-method for member variable
	 * return litType
	 */
	public String getLitType(){
			  return litType;
	}

	/**
	 * Get-method for member variable
	 * return litTheme
	 */
	public String getLitTheme(){
			  return litTheme;
	}

	/**
	 * Get-method for member variable
	 * return litCause
	 */
	public String getLitCause(){
			  return litCause;
	}

	/**
	 * Set-method for member variable
	 * @param s the (literal) type to be set
	 */
	public void setLitType(String s){
			  litType = s;
	}

	/**
	 * Set-method for member variable
	 * @param s the (literal) theme to be set
	 */
	public void setLitTheme(String s){
			  litTheme = s;
	}

	/**
	 * Set-method for member variable
	 * @param s the (literal) cause to be set
	 */
	public void setLitCause(String s){
			  litCause = s;
	}

	/**
	 * Set-method for member variable
	 * @param a the parent abstract to be set
	 */
	public void setAbstract(TQLAbstract a){
		parent_abstract = a;
	}

	/**
	 * Method for returning an XML-representation of this event. Used for getting "distilled" events to write to out-files.
	 * @see XMLWriter
	 */
	public String getXML(){
		if ( id > 0 ) {
			return "<event id='"+id+"' sentence='"+parent.input+"' abstract='"+parent.parent_abstract.medlineid+"'>\t<type class='"+this.getType()+"' />\t<theme idref='"+this.getTheme()+"' />\t<cause idref='"+this.getCause()+"' /></event>\n";
		} else {
			return "<event id='0' />\n";
		}
	}


}
