import java.util.ArrayList;
import java.util.Hashtable;

/**
 * Class to hold GEvents, GSentences, sentence counter and a hash table over terms.
 * @see GEvent
 * @see GSentence
 * @see Hashtable
 *
 */
public class GAbstract {

	public String PMID;
	public ArrayList<GEvent> Gevents = new ArrayList<GEvent>();
	public ArrayList<GSentence> Gsentences = new ArrayList<GSentence>();
	public Hashtable<String,String> ht_lex = new Hashtable<String,String>();
	public Hashtable<String,String> ht_sem = new Hashtable<String,String>();
	public int sentence_counter = 1;

	/**
	 * This class does not need anything to be instantiated.
	 */
	public GAbstract () {
	}

	/**
	 * Adding asentence to the array of sentences and incrementing the counter.
	 * @param gs The GSentence to be added
	 */
	public void addSentence (GSentence gs) {
			  Gsentences.add(gs);
			  gs.sentence_number = sentence_counter++;
	}
	/**
	 * Adding an event to the array of events, and setting this abstract as parent for the GEvent.
	 * @param ge The event to be added
	 */
	public void addEvent(GEvent ge) {
			  ge.setParent(this);
			  Gevents.add(ge);
	}

	/**
	 * Setting the unique medline identification number for this abstract.
	 * @param s the Medline IDentificator
	 */
	public void setPMID (String s) {
			  PMID = s;
	}

	/**
	 * Returns the size of this abstracts event array.
	 * @return the number of events in this abstract
	 */
	public int countEvents(){
			  return Gevents.size();
	}

	/**
	 * Returns the number of sentences in this abstract.
	 * @return number of sentences in this abstract.
	 */
	public int countSentences(){
			  return Gsentences.size();
	}

	/**
	 * Tries to look up the lexical interpretation of a term code in this abstracts hashtable.
	 * @param term the term to be looked up
	 * @return the lexical representation for this term
	 */
	public String resolveTermLex(String term){
		return ht_lex.get(term);
	}

	/**
	 * Traverses the list of Terms in each Sentence, and adds key,value-pair to this abstracts hashtable. It also calls resolveTermPermanent() on each GEvent.
	 */
	public void buildTerms(){
		for( GSentence gs : Gsentences ) {
			for( GTerm gt : gs.terms ) {
				// if ( ( gt.id == null ) || ( gt.lex == null ) || ( gt.sem == null ) ) System.err.println("== WARNING: ignoring incomplete term: ("+gt.id+","+gt.lex+","+gt.sem+")");
				if( gt.lex != null ) ht_lex.put(gt.id,gt.lex);
				if( gt.sem != null ) ht_sem.put(gt.id,gt.sem);
			}
		}

		for ( GEvent ge : Gevents ) {
			ge.resolveTermPermanent();
		}
	}

	/**
	 * Returns this abstracts list of events.
	 * return an arraylist of events
	 */
	public ArrayList getEvents(){
		return Gevents;
	}

	/**
	 * Get the theme of an event, given it's PMID. If the PMID does not exist, it returns null.
	 * @param eventID the unique identification number for this event
	 * @return theme of the event or null
	 */
	public String getEventTheme(String eventID){
		for(GEvent ge : Gevents){
			if(ge.id == eventID) return ge.get("theme");
		}
		return null;
	}

	/**
	 * Get the type of an event, given it's PMID. If the PMID does not exist, it returns null.
	 * @param eventID the unique identification number for this event
	 * @return type of the event or null
	 */
	public String getEventType(String eventID){
		for(GEvent ge : Gevents){
			if( (ge.id != null ) && ge.id.equals(eventID)) {
				return ge.get("typeclass");
			}
		}
		return null;
	}

	/**
	 * Get the sentences in this abstract.
	 * @return the sentences in this abstract
	 */
	public ArrayList getSentences(){
		return Gsentences;
	}

	/**
	 * Method to echo the contents of this abstract to stdout.
	 */
	public void hello() {
		System.out.println("Abstract PMID: "+PMID);
		for(GSentence gs : Gsentences){
			gs.hello();
		}
		for(GEvent ge : Gevents){
			ge.hello();
		}
	}

	/**
	 * Find the most likely resolve of a reference (term, event theme or semantic expression in prioritized sequence)
	 * @param ref the reference value
	 * @return the resolved reference or null
	 */
	public String resolveReference(String ref){
		if(ref == null) System.err.println("== WARNING: trying to resolve nullstring");
		String resolve = null;

		// try term
		resolve = resolveTermLex(ref);

		if (resolve != null) return resolve;

		// and event
		resolve = getEventType(ref);

		if (resolve != null) return resolve;

		// no
		return ref;
			
	}


}
