import java.util.Hashtable;
import java.util.Enumeration;

/**
 * A container class for holding all data on events from the golden corpus.
 * Attributes are located in a hashtable, which set- and get-methods should be employed to access.
 */
public class GEvent {

	public String id;
	public Hashtable<String,String> ht = new Hashtable<String,String>();
	public GAbstract parent;
	public String sentence;

	/**
	 * No data is needed to construct an GEvent
	 */
	public GEvent(){
	}

	/**
	 * Sets the of parent abstract this Event
	 * @param p the parent abstract
	 */
	public void setParent(GAbstract p){
		parent = p;
	}

	/**
	 * Sets the identification string from corpus of this Event
	 * @param i identification string from corpus
	 */
	public void setID(String i){
		id = i;
	}

	/**
	 * Sets the type of this Event
	 * @param value event type
	 */
	public void setType(String value){
		ht.put("typeclass",value);
	}

	/**
	 * Sets the theme of this Event
	 * @param value event theme
	 */
	public void setTheme(String value){
		ht.put("theme",value);
	}

	/**
	 * Sets the cause of this Event
	 * @param value event cause
	 */
	public void setCause(String value){
		ht.put("cause",value);
	}

	/**
	 * Sets an attribute of this Event
	 * @param key the attribute name
	 * @param value the attribute value
	 */
	public void set(String key, String value){
		ht.put(key,value);
	}

	/**
	 * Gets an attribute value from this Event
	 * @return the value associated with an attribute
	 */
	public String get(String key){
		return ht.get(key);
	}

	/**
	 * Gets the type of this Event
	 * @return event type
	 */
	public String getType(){
		return ht.get("typeclass");
	}

	/**
	 * Gets the theme of this Event
	 * @return event theme
	 */
	public String getTheme(){
		return ht.get("theme");
	}

	/**
	 * Gets the cause of this Event
	 * @return event cause
	 */
	public String getCause(){
		return ht.get("cause");
	}

	/**
	 * Gets the real (resolved) type of this Event
	 * @return real event type
	 * @see #resolveTermPermanent()
	 */
	public String getRealType(){
		return ht.get("real_typeclass");
	}

	/**
	 * Gets the real (resolved) theme of this Event
	 * @return real event theme
	 * @see #resolveTermPermanent()
	 */
	public String getRealTheme(){
		return ht.get("real_theme");
	}

	/**
	 * Gets the real (resolved) of this Event
	 * @return real event cause
	 * @see #resolveTermPermanent()
	 */
	public String getRealCause(){
		return ht.get("real_cause");
	}

	/**
	 * Gets the XML-representation of this Event for printing distilled 
	 * @return a string representing the essentials of this event.
	 */
	public String getXML(){
		if ( id != null ) {
			return "<event id='"+id+"' sentence='"+sentence+"' abstract='"+parent.PMID+"'>\t<type class='"+getRealType()+"' />\t<theme idref='"+getRealTheme()+"' />\t<cause idref='"+getRealCause()+"' /></event>\n";
		} else {
			return "<event id='0' />\n";
		}
	}

	/**
	 * Method to resolve each attribute into the lexiacal representation, and store is as real_attribute.
	 */
	public void resolveTermPermanent() {
		String theme, cause;
		// typeclass is always correct
		ht.put("real_typeclass",ht.get("typeclass"));
		// theme may be a reference
		if ( ( theme = ht.get("theme") ) != null ) ht.put("real_theme",parent.resolveReference(theme));
		// cause may be a reference
		if ( ( cause = ht.get("cause") ) != null ) ht.put("real_cause",parent.resolveReference(cause));

	}

	/**
	 * Method for echoing the content of this event to stdout.
	 */
	public void hello() {
		String theme, cause;
		System.out.println("== Event ID: "+id);
		System.out.println(" typeclass: "+ht.get("real_typeclass"));
		System.out.println(" theme: "+ht.get("real_theme"));
		System.out.println(" cause: "+ht.get("real_cause"));
	}

}
