import java.util.ArrayList;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.util.Hashtable;	
import org.apache.commons.collections.map.MultiValueMap;

/**
 * An abstraction class for holding synonym definitions. 
 *
 * Constructed to read the file "syno.dat", which is a CSV-formatted file containg key,value-pairs.
 * The contents of this file is availiable for getting and setting via public methods.
 *
 * This class employs the MultiValueMap from org.apache.commons.collections.map to be able to map one word to different synonymes.
 *
 * @author Harald Svik
 * @see Anto
 *
 */

public class Syno {

	public File synofile;
	public BufferedReader d;
	public MultiValueMap ht = new MultiValueMap();


	/**
	 * Constructor for creating an Syno. Tries to access the file "syno.dat".
	 * If the files does not exist, it is created with default permissions. (Set by umask in UNIX environment)
	 *
	 */
	public Syno () {

		try { 
			this.synofile = new File("syno.dat");
			if ( ! synofile.exists() ) synofile.createNewFile();
			d = new BufferedReader(new FileReader(synofile));

			while (d.ready()) {
				String line = d.readLine();
				String[] parts = line.split(",",2);
				ht.put(parts[0],parts[1]);
			}

			System.out.println("== building of syno.dat complete ("+ht.totalSize()+" entries)");
			d.close();

		} catch ( Exception e ) {
			e.printStackTrace();
			System.out.println("== ERROR: could not open syno.dat");
			System.exit(1);
		}

	}

	/**
	 * Method to check if a word is an synonyme of another.
	 * Looks up all synonymes to the word in x, and then traverses all synonyme definitions.
	 * Returns true if Y is found. That is, true is the words are synonymes, false if the pair is not defined.
	 *
	 * @param x	the (gold) term to look up
	 * @param y the (test) term to compare
	 * @return true if the words are defined synonymes, else false
	 */
	public boolean isSyno(String x, String y){

		if ( ht.size(x) == 0) return false;
				  
		ArrayList<String> c = (ArrayList) ht.getCollection(x);
		// System.out.println(x+" collection is of size "+c.size());

		for ( String s : c ) {
			// System.out.println(x+" == "+s);
			if ( s.equals(y) ) return true;
		}

		// redundant
		return false;

	}

	/**
	 * Method to define two words as synonymes. Their sequence is not abitary: the first parameter is from GOLD and the second is from TEST.
	 * Writes (appends) the result to syno.dat, and assumes that the file is availiable. (If it was not, there would not have been an instance of Synoavailiable.)
	 *
	 * @param x the GOLD word
	 * @param y the TEST word
	 *
	 */

	public void setSyno(String x, String y){

		try {

			FileWriter fw = new FileWriter(synofile,true); // true == append
			String towrite = x+","+y+"\n";
			// System.out.println("write: "+towrite);
			fw.write(towrite);
			fw.flush();
			fw.close();

		} catch ( Exception e ) {

			e.printStackTrace();
			System.out.println("== ERROR: failed to write to syno.dat");
			System.exit(1);

		}

		ht.put(x,y);

	}


}
