import java.util.ArrayList;
import java.util.regex.*;
import java.util.HashMap;


/**
 * This class contains most of the logic for recognising events. It takes care of reading all the lines in a block of statement, and process them according to rules. It constructs an ArrayList of Events, and saves the Events redundantly to the parent abstract.
 *
 * @see TQLAbstract
 * @see OntologyParser
 *
 */
public class TQL {

	public OntologyParser op;
	public TQLAbstract parent_abstract;
	public ArrayList<String> statements = new ArrayList<String>();
	public ArrayList<Event> events = new ArrayList<Event>();
	public HashMap<String, String> nouns = new HashMap<String, String>();
	public int input;
	public int block;
	public int eventcounter = 0;

	public final boolean debug_tql = false;

	/**
	 * A TQL represents a block of TQL-statements, and the associated events.
	 *
	 * @param input the sentence number 
	 * @param block number of successfully parsed sentences including this one
	 * @param par parent abstract
	 * @param op the ontologyparser used to determine significance of words
	 */
	public TQL(int input, int block, TQLAbstract par, OntologyParser op){
		// System.out.println("Created new instance of TQL("+input+","+block+")");
		this.input = input;
		this.block = block;
		this.parent_abstract = par;
		this.op = op;
	}

	/**
	 * Add a TQL-statement to the list of lines.
	 * @param line the statement to be added
	 */
	public void addLine(String line){
		// System.out.println("added: "+line);
		//
		statements.add(line);
	}

	/**
	 * Get an arraylist of all statements
	 * @return an arraylist of all statements
	 */
	public ArrayList getLines(){
		return statements;
	}

	/**
	 * A wrapper method for printEvents
	 * @see #printEvents()
	 */
	public void hello(){
		// System.out.println("== TQL("+input+","+block+")");
		// for(String s : statements) System.out.println(s);
		printEvents();
	}

	/**
	 * Get an arraylist of the events in this TQL
	 * @return an arraylist of events in this tql
	 */
	public ArrayList getEvents(){
		return events;
	}

	/**
	 * Echo the events in this TQL to stdout
	 */
	public void printEvents(){
		for(Event i : events){
			i.printEvent();
		}
	}

	/**
	 * Get the next available event id number
	 * @return event id number
	 */
	public synchronized int getNextEvID(){
			  eventcounter++;
			  return eventcounter;
	}

	/**
	 * initiate extraction of events accorting to rules specified in this funciton
	 * @see #findNoun()
	 * @see #findVerb()
	 * @see #findEvent()
	 */
	public void extractEvents(){

		// PUT INITIATING RULE CALLS HERE
		// findX = look up initiation of events
		// identify = find attributes to an existing event
		resolveSkolemns();	// rule 0 - skolemns may have conflicting sequence

		findNoun(); // rule 1

		findVerb(); // rule 2

		findEvent(); // rule 3
		
	}

	/**
	 * initiate extraction of events according to rule
	 * @see #identifyNoun(String)
	 */
	public void findNoun(){	// RULE 1: if noun isa activity -> activity isa event
		for(String s : statements){
			identifyNoun(s);	
		}
	}

	/**
	 * initiate extraction of events according to rule
	 * @see #identifyTransVerb(String)
	 * @see #identifyITVerb(String)
	 */
	public void findVerb(){	// induce/lipopolysaccharide/phosphorylation/sk(1)
		for(String s : statements){
			identifyTransVerb(s);	
		}

		for(String s : statements){
			identifyITVerb(s);	
		}
	}

	/**
	 * initiate extraction of events according to rule
	 * @see #identifyEvent(String)
	 */
	public void findEvent() { // srel/to/thing/activation/sk(416) // event/real/sk(416)
		for(String s : statements){
			identifyEvent(s);	
		}
	}


	/**
	 * map all skolemns in this TQL to their appropriate class
	 */
	public void resolveSkolemns(){
		Pattern p_1 = Pattern.compile("^([\\w()]+)\\s?isa\\s(\\w*)$");

		for(String s : statements){

			Matcher m = p_1.matcher(s);	

			while ( m.find() ) {
				String g1 = m.group(1).trim();
				String g2 = m.group(2).trim();
				nouns.put(g1, g2);
			}
		}
	}

	/**
	 * check if a sentence contains a noun and initiate event logic. 
	 * if it matches an event rule, create an event, and find more data on the event
	 * @param s the line of tql code
	 * @see #findNREL(Event)
	 * @see #identifyAdjType(Event)
	 */
	public void identifyNoun(String s){
		Pattern p_1 = Pattern.compile("^([\\w()]+)\\s?isa\\s(\\w*)$");
		Matcher m = p_1.matcher(s);	

		while ( m.find() ) {
			String g1 = m.group(1).trim();
			String g2 = m.group(2).trim();


			// if( op.isInOntology( g2 ) ) {
			if( op.isInOntology(g2) /* || op.isOfClass(g2,"Biological_process") */ ) {
				// identified a process etc - create event and find theme denoted by induce/lipopolysaccharide/phosphorylation/sk(1)
				if ( debug_tql ) System.out.println("MATCH NOUN: "+g2+" in "+s+" with regexp "+p_1.pattern());

				Event e = new Event(this);	
				e.setType(sk(g1));
				e.setLitType(g1);
				events.add(e);
				parent_abstract.addEvent(e);
				findNREL(e);
				identifyAdjType(e);
				if ( debug_tql ) e.hello();
			}
		}
	}

	
	/**
	 * check if a sentence contains a verb, and initiate event logic.
	 * if it matches an event rule, create an event, and find more data on the event
	 * @param s the line of tql code
	 * @see #identifyAdjTheme(Event)
	 */
	public void identifyTransVerb(String s){
		Pattern p_4 = Pattern.compile("^(\\w+)/([\\w()]+)/([\\w()]+)/sk\\(\\d+\\)$");
		Matcher m = p_4.matcher(s);	

		// System.out.println(s+" for TRANS with regexp "+p_4.pattern());

		while ( m.find() ) {
	  

			String g1 = m.group(1).trim();
			String g2 = m.group(2).trim();
			String g3 = m.group(3).trim();

			if( op.isInOntology( g1 ) ) {

				if ( debug_tql ) System.out.println("MATCH TR-VERB: "+s+" with regexp "+p_4.pattern());

				Event e = new Event(this, g1);	
				e.setType(g1);
				e.setCause(sk(g2));
				e.setLitCause(g2);
				e.setTheme(sk(g3));
				e.setLitTheme(g3);
				identifyAdjTheme(e);
				events.add(e);
				parent_abstract.addEvent(e);
				if ( debug_tql ) e.hello();
			}
		}
	}

	/**
	 * check if a sentence contains a event definition, and initiate further logic.
	 * if it matches an event rule, create an event, and find more data on the event
	 * @param s the line of tql code
	 * @see #identifySREL2(Event,String)
	 */
	public void identifyEvent(String s){
		Pattern p_4 = Pattern.compile("^event/real/sk\\((\\d+)\\)+$"); // event/real/sk(416)
		Matcher m = p_4.matcher(s);	

		// System.out.println(s+" for EVENT with regexp "+p_4.pattern());

		while ( m.find() ) {

			String sk = m.group(1).trim();

				if ( debug_tql ) System.out.println("MATCH EVENT: "+s+" with regexp "+p_4.pattern());

				Event e = new Event(this);	
				identifySREL2(e,sk);
				if ( e.getType() != null ) {
					events.add(e);
					parent_abstract.addEvent(e);
					if ( debug_tql ) e.hello();
				}
		}
	}


	/**
	 * check if a sentence contains an intrasitive verb, and possibly initiate event
	 * @param s the line of tql code
	 * @see #identifySREL(Event)
	 */
	public void identifyITVerb(String s){

		Pattern p_5 = Pattern.compile("^(\\w+)/([\\w()]+)/sk\\(\\d+\\)$");
		Matcher m = p_5.matcher(s);	

		// System.out.println(s+" for INTRANS with regexp "+p_5.pattern());

		while ( m.find() ) {

			String g1 = m.group(1).trim();
			String g2 = m.group(2).trim();

			if( op.isBuzzVerb( g1 ) ) {

				if ( debug_tql ) System.out.println("MATCH IT-VERB: "+s+" with regexp "+p_5.pattern());

				Event e = new Event(this, g1);	
				e.setTheme(sk(g2));
				e.setLitTheme(g2);
				identifySREL(e);
				events.add(e);
				parent_abstract.addEvent(e);
				if ( debug_tql ) e.hello();
			}
		}
	}

	// NLP-feature 
	public String prolog(String s){
		if( s == null ) {
			return "(\\w+)";
		} else {
			return s;
		}
	}

	public String sk(String s){
		if( s == null ) {
			System.out.println("== WARNING: tried to resolve empty skolemn string. May be a bug."); 
		 	return null; // for sikkerhets skyld
		}

		Pattern sk = Pattern.compile("^sk\\(\\d+\\)$");
		Matcher msk = sk.matcher(s);
		if ( msk.find() ) {
		 	String nounclass = nouns.get( msk.group() );
		 	return nounclass;
		} else {
		 	return s;
		}
	}


	/**
	 * check if a sentence contains a noun relation, and possibly add into to already existing event
	 * (this method wraps for identifyNREL()
	 * @param e the event to add data to
	 * @see #identifyNREL(Event)
	 */
	public void findNREL(Event e){

		//boolean found;

		identifyNREL(e);
		//for(String s : statements){
		//	found = identifyNREL(e, s);	
		//	if ( found ) break;
		//}
	}

	/**
	 * check if a sentence contains a noun relation, and possibly add into to already existing event
	 * @param e the event to add data to
	 * @see #identifyAdjTheme(Event)
	 */
	public void identifyNREL(Event e){
			  // 
		// dersom referanse er en skolemn m vi escape parenteser
		String ref = e.getLitType();
		Pattern p_2 = Pattern.compile("sk\\((\\d+)\\)");
		Matcher l = p_2.matcher(ref);	
		if ( l.find() ) {
			ref = "sk\\("+l.group(1)+"\\)";
		} 

		if ( ref.equals(")") || ref.equals("(") ) ref = ".*";
		Pattern p_3 = Pattern.compile("^nrel/of/(\\w+)/(\\w+)/"+ref+"/([\\w()]+)$");

		for(String s : statements) {
			// System.out.println(s+" with regexp "+p_3.pattern());
			Matcher m = p_3.matcher(s);	
			if ( m.find() ) {
				if ( debug_tql ) System.out.println("MATCH NREL: "+s+" with regexp "+p_3.pattern());
				e.setTheme(sk(m.group(3).trim()));			
				e.setLitTheme(m.group(3).trim());			
				identifyAdjTheme(e);
			} 
		}

		Pattern p_4 = Pattern.compile("^nrel/by/(\\w+)/(\\w+)/"+ref+"/([\\w()]+)$");

		for(String s : statements) {
			// System.out.println(s+" with regexp "+p_4.pattern());
			Matcher n = p_4.matcher(s);	
			if ( n.find() ) {
				if ( debug_tql ) System.out.println("MATCH NREL: "+s+" with regexp "+p_4.pattern());
				e.setCause(sk(n.group(3).trim()));			
				e.setLitCause(n.group(3).trim());			
			} 
		}

		// litt usikker p denne 
		Pattern p_5 = Pattern.compile("^nrel/for/(\\w+)/(\\w+)/([\\w()]+)/"+ref+"$");

		for(String s : statements) {
			//System.out.println(s+" with regexp "+p_5.pattern());
			Matcher n = p_5.matcher(s);	
			if ( n.find() ) {
				if ( debug_tql ) System.out.println("MATCH NREL: "+s+" with regexp "+p_5.pattern());
				e.setCause(sk(n.group(3).trim()));			
				e.setLitCause(n.group(3).trim());			
			} 
		}

		Pattern p_6 = Pattern.compile("^nrel/to/(\\w+)/(\\w+)/"+ref+"/([\\w()]+)$");

		for(String s : statements) {
			// System.out.println(s+" with regexp "+p_4.pattern());
			Matcher n = p_6.matcher(s);	
			if ( n.find() ) {
				if ( debug_tql ) System.out.println("MATCH NREL: "+s+" with regexp "+p_6.pattern());
				e.setCause(sk(n.group(3).trim()));			
				e.setLitCause(n.group(3).trim());			
			} 
		}

	}

	public void identifySREL(Event e){

		Pattern p_3 = Pattern.compile("^srel/to/thing/([\\w()]+)/[\\w()]+$");

		for(String s : statements) {
			// System.out.println(s+" with regexp "+p_3.pattern());
			Matcher m = p_3.matcher(s);	
			if ( m.find() ) {
				if ( debug_tql ) System.out.println("MATCH SREL: "+s+" with regexp "+p_3.pattern());
				e.setCause(sk(m.group(1).trim()));			
				e.setLitCause(m.group(1).trim());			
				identifyAdjTheme(e);
			} 
		}
	}

	public void identifySREL2(Event e, String skNum){

		Pattern p_3 = Pattern.compile("^srel/to/thing/(\\w+)/sk\\("+skNum+"\\)$");

		for(String s : statements) {
			// if ( debug_tql ) System.out.println(s+" with regexp "+p_3.pattern());
			Matcher m = p_3.matcher(s);	
			if ( m.find() ) {
				if ( debug_tql ) System.out.println("MATCH SREL2: "+s+" with regexp "+p_3.pattern());
				String type = m.group(1).trim();			
				if ( op.isInOntology(type) ) {
					e.setType(type);
					identifyAdjTheme(e);
				}
			} 
		}
	}

	public void identifyAdjType(Event e){
		//System.out.println("== HERE ==");

		String skolemn = escapeRegEx(e.getLitType());

		Pattern p_7 = Pattern.compile("^adj/(\\w+)/"+skolemn+"/real$");

		for(String s : statements) {
			// System.out.println(s+" with regexp "+p_7.pattern());
			Matcher n = p_7.matcher(s);	
			while ( n.find() ) {
				String adj = n.group(1).trim();
				// if ( op.isBuzzAdjective(adj) ) {
					if ( debug_tql ) System.out.println("MATCH ADJ (type): "+s+" with regexp "+p_7.pattern());
					e.type_extended = adj.concat(" "+e.type);
				// } 
			} 
		}


		//System.out.println("== DONE ==");
	}


	/**
	 * check if a sentence contains an adjective that could add data to an event 
	 * @param e the event to add data to
	 */
	public void identifyAdjTheme(Event e){
		//System.out.println("== HERE ==");

		String skolemn = escapeRegEx(e.getLitTheme());

		Pattern p_7 = Pattern.compile("^adj/(\\w+)/"+skolemn+"/real$");

		for(String s : statements) {
			// System.out.println(s+" with regexp "+p_7.pattern());
			Matcher n = p_7.matcher(s);	
			// while (all) , if (one) - take only one adjective
			while ( n.find() ) {
				if ( debug_tql ) System.out.println("MATCH ADJ (theme): "+s+" with regexp "+p_7.pattern());
				String adj = n.group(1).trim();
				// if ( op.isBuzzAdjective(adj) ) {
				e.cause_adjectives += adj + " ";
				e.theme_extended = e.cause_adjectives + e.theme; 
				//}
			} 
		}


		//System.out.println("== DONE ==");
	}

	/**
	 * for an abitary string, escape each character that could be misinterpreted in the context of regular expressions
	 * @param s the string to be escaped
	 * @return an escaped string
	 */
	public String escapeRegEx(String s){

		if ( s == null ) return ".*?";

		String result = "";

		for ( int i = 0 ; i < s.length() ; i++ ) {
			String c = s.substring(i,i+1);
			if ( c.equals("(") ) {
				result = result.concat("\\(");
			} else if ( c.equals(")") ) {
				result = result.concat("\\)");
			} else {
				result = result.concat(c);
			}	
		}
		return result;
	}

}
